class FbInput:
    @staticmethod
    def create_text_message(text, quick_reply_payload=None):
        sender_id = 358
        recipient_id = 588667681307355
        message = {
            'seq': 53,
            'mid': 'mid.1466750926480:b2598e311e5ad42d71',
            'text': text
        }
        if quick_reply_payload:
            message.update({
                'quick_reply': {
                    'payload': quick_reply_payload
                }
            })
        template = {
            'sender': {
                'id': sender_id
            },
            'message': message,
            'recipient': {
                'id': recipient_id
            },
            'timestamp': 1466750926487
        }

        return template

    @staticmethod
    def create_attachment_message():
        sender_id = 358
        recipient_id = 588667681307355
        template = {
            'sender': {
                'id': sender_id
            },
            'recipient': {
                'id': recipient_id
            },
            'message': {
                'mid': 'mid.1466751206432:f290a56f73f8b02b63',
                'seq': 55,
                'sticker_id': 369239263222822,
                'attachments': [{
                    'type': 'image',
                    'payload': {
                        'url': 'https://example.com/image.png'
                    }
                }]
            },
            'timestamp': 1466750926487}

        return template

    @staticmethod
    def create_delivery():
        sender_id = 358
        recipient_id = 588667681307355
        template = {
            'sender': {
                'id': sender_id
            },
            'recipient': {
                'id': recipient_id
            },
            'delivery': {
                'seq': 12,
                'mids': ['mid.1466675298251:7204294a89a4aae828'],
                'watermark': 1466675298280
            },
            'timestamp': 0}

        return template

    @staticmethod
    def create_attachment_message_with_coordinates():
        template = {
            'timestamp': 1472722892136,
            'message': {
                'attachments': [{
                    'payload': {
                        'coordinates': {
                            'lat': 52.440433969359,
                            'long': 16.912738256133
                        }
                    },
                    'url': 'https://www.facebook.com/l.php?u=https%3A%2F%2Fwww.bing.com%2Fmaps%2Fdefault.aspx%3Fv%3D2%26pc%3DFACEBK%26mid%3D8100%26where1%3DPi%25C4%2585tkowska%2B200%252C%2B61-691%2BPoznan%252C%2BPoland%26FORM%3DFBKPL1%26mkt%3Den-US&h=gAQExQLq5&s=1&enc=AZMaa60a3PRmvr7cCq_NxbB0_9bDCkvMtjEyVPUXdI8k_Fn_Dc3azA1N38Rf1gALRVZb8Z63q1iL3iKUXyp3h7attTPdIn1eM3Vsg6zjrFutHw',
                    'title': 'Foo location',
                    'type': 'location'
                }],
                'mid': 'mid.1472722891670:f5daf1db754cb91616',
                'seq': 18
            },
            'sender': {
                'id': '1072879492798443'
            },
            'recipient': {
                'id': '319022018450028'
            }
        }

        return template

    @staticmethod
    def create_postback(payload):
        sender_id = 358
        recipient_id = 588667681307355
        template = {
            'sender': {
                'id': sender_id
            },
            'recipient': {
                'id': recipient_id
            },
            'postback': {
                'payload': payload
            },
            'timestamp': 1466585047605}

        return template

    @staticmethod
    def create_read():
        template = {
            'read': {
                'watermark': 1467892173172,
                'seq': 537
            },
            'timestamp': 1467892190179,
            'sender': {
                'id': '1129675700423442'
            },
            'recipient': {
                'id': '588667681307355'
            }
        }

        return template

    @staticmethod
    def create_not_supported():
        template = {
            'sender': {
                'id': 1
            },
            'not_supported': {
                'seq': 53,
                'mid': 'mid.1466750926480:b2598e311e5ad42d71',
                'text': 'text'
            },
            'recipient': {
                'id': 1
            },
            'timestamp': 1466750926487}

        return template



import dependency_injector.containers as containers
import dependency_injector.providers as providers

from messenger_sdk.config import Config
from messenger_sdk.api_client import ApiClient
from messenger_sdk.events import EventFactory
from messenger_sdk.fb_manager import FbManager
from messenger_sdk.log import Log
from messenger_sdk.handler import BaseHandler
from messenger_sdk.middlewares.sender_action import TypingOnAction


class BaseContainer(containers.DeclarativeContainer):
    config = providers.Singleton(Config)
    api_client = providers.Factory(ApiClient)
    event_factory = providers.Singleton(EventFactory)
    log = providers.Singleton(Log, config)
    fb_manager = providers.Factory(FbManager, api_client, config)

    typing_on_action = providers.Factory(TypingOnAction, config)

    base_middlewares = [
        {'class': typing_on_action, 'priority': 90},
    ]

    base_request_handler = providers.Factory(BaseHandler,
                                             event_factory=event_factory,
                                             fb_manager=fb_manager,
                                             base_middlewares=base_middlewares)



import logging

from tornado import gen

from messenger_sdk.fb_response import FbResponse
from messenger_sdk.templates import Response
from messenger_sdk.fb_manager import FbManager
from messenger_sdk.events import EventFactory
from messenger_sdk.middlewares.middleware import Middleware, EventMiddleware, ResponseMiddleware


class BaseHandler:
    _custom_middlewares = list()

    def __init__(self, event_factory: EventFactory, fb_manager: FbManager, base_middlewares: list):
        self._event_factory = event_factory
        self._fb_manager = fb_manager
        self._base_middlewares = base_middlewares

    @property
    def custom_middlewares(self):
        return self._base_middlewares

    @custom_middlewares.setter
    def custom_middlewares(self, middlewares: list):
        self._custom_middlewares = middlewares

    @gen.coroutine
    def handle_entries(self, entries):
        for event in entries:
            try:
                logging.info('Input event: %s', event)
                event = self._event_factory.create_event(event)
                response = FbResponse()
                middleware_classes = self.load_middleware_classes()
                yield self.process_middlewares(middleware_classes=middleware_classes, event=event, response=response)
            except Exception as err:
                logging.error('Unable to handle event: %s', err)
                logging.debug('<--- TYPING OFF --->')
                response = FbResponse(Response(recipient_id=event.recipient_id,
                                               sender_action=Response.SENDER_ACTION_TYPING_OFF))
                yield self._fb_manager.send(response)

    @gen.coroutine
    def process_middlewares(self, middleware_classes, event, response):
        middleware_classes_count = len(middleware_classes)
        for index, middleware_class in enumerate(middleware_classes):
            if not event.propagate:
                break
            yield self._process_middleware(event=event, response=response, middleware_class=middleware_class)
            if response.send or index == middleware_classes_count - 1:
                yield self._fb_manager.send(response=response)
                response = FbResponse()

    @staticmethod
    def check_middleware_classes_list(middleware_classes: list):
        if len(middleware_classes) == 0:
            raise ValueError('Middleware classes list cannot be empty.')

        return True

    def load_middleware_classes(self):
        middlewares = self._base_middlewares + self._custom_middlewares
        self.check_middleware_classes_list(middlewares)

        return sorted(middlewares, key=lambda k: k['priority'], reverse=True)

    def check_mw_type(self, mw_instance):
        if not isinstance(mw_instance, Middleware):
            raise TypeError('Expected {expected} class, {given} given in {self_name}.'.format(
                expected=Middleware.__class__.__name__,
                given=type(mw_instance).__name__,
                self_name=self.__class__.__name__))

        return True

    @gen.coroutine
    def _process_middleware(self, event, response, middleware_class):
        mw_instance = middleware_class.get('class')()
        self.check_mw_type(mw_instance)
        if not any(isinstance(event, supported_event) for supported_event in mw_instance.supported_events):
            return
        logging.debug('<--- Middleware: {middleware_name} --->'.format(middleware_name=mw_instance.__class__.__name__))
        if isinstance(mw_instance, EventMiddleware):
            logging.debug('Executing: {function_name}()'.format(function_name=mw_instance.process_event.__name__))
            yield mw_instance.process_event(event=event)
        if isinstance(mw_instance, ResponseMiddleware):
            logging.debug('Executing: {function_name}()'.format(function_name=mw_instance.process_response.__name__))
            yield mw_instance.process_response(event=event, response=response)



from abc import ABCMeta, abstractmethod

from messenger_sdk import events
from messenger_sdk.fb_response import FbResponse


class Middleware(metaclass=ABCMeta):
    @property
    def supported_events(self):
        return [events.MessageEvent, events.PostbackEvent]


class EventMiddleware(Middleware):
    @abstractmethod
    def process_event(self, event: events.Event):
        raise NotImplementedError()


class ResponseMiddleware(Middleware):
    @abstractmethod
    def process_response(self, event: events.Event, response: FbResponse):
        raise NotImplementedError()



from unittest.mock import MagicMock

from tornado.concurrent import Future
from tornado.testing import AsyncTestCase

from messenger_sdk.container import BaseContainer
from messenger_sdk.fb_input import FbInput
from messenger_sdk.sample_response import BaseSampleTemplateResponse


class MessengerAsyncTestCase(AsyncTestCase):
    def setUp(self):
        super().setUp()
        self._fb_input = FbInput()
        self._event_factory = BaseContainer.event_factory()
        self._sample_template_response = BaseSampleTemplateResponse()

    @staticmethod
    def get_async_function_mock(return_value):
        future = Future()
        future.set_result(return_value)
        return MagicMock(return_value=future)
